@@ -5,6 +5,7 @@ use url::Host;
 use http::uri::Scheme;
 use std::fmt;
 use std::error::Error;
+use std::net::SocketAddr;
 
 #[derive(Debug,Clone,PartialEq)]
 pub enum Nodename {
@@ -12,6 +13,18 @@ pub enum Nodename {
     Obf(String),
 }
 
+impl From<IpAddr> for Nodename {
+    fn from(ip: IpAddr) -> Self {
+        Nodename::Ip(ip)
+    }
+}
+
+impl From<String> for Nodename {
+    fn from(obf: String) -> Self {
+        Nodename::Obf(obf)
+    }
+}
+
 impl fmt::Display for Nodename {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -59,6 +72,18 @@ pub enum Nodeport {
     None
 }
 
+impl From<u16> for Nodeport {
+    fn from(ui: u16) -> Self {
+        Nodeport::Port(ui)
+    }
+}
+
+impl From<String> for Nodeport {
+    fn from(obf: String) -> Self {
+        Nodeport::Obf(obf)
+    }
+}
+
 impl fmt::Display for Nodeport {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -89,6 +114,21 @@ pub enum Node {
     Unknown
 }
 
+impl From<SocketAddr> for Node {
+    fn from(saddr:  SocketAddr) -> Self {
+        Node::Node(saddr.ip().into(),saddr.port().into())
+    }
+}
+
+impl From<Option<SocketAddr>> for Node {
+    fn from(saddr:  Option<SocketAddr>) -> Self {
+        if let Some(saddr) = saddr {
+            return  Node::Node(saddr.ip().into(),saddr.port().into())
+        }
+        Node::Unknown
+    }
+}
+
 impl fmt::Display for Node {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -307,11 +347,12 @@ impl FromStr for Forwarded {
     //TODO. make it more powerfull
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let mut v = Vec::new();
+        if s.is_empty() == false {
         for e in s.split(',') {
             let elm = ForwardedElement::from_str(e)
                 .map_err(|_| ParseForwardedError)?;
             v.push(elm);
-        }
+        }}
         return Ok(Forwarded {
             forwarded_element:  v
         });
@@ -344,6 +385,45 @@ impl Forwarded {
     pub fn remove_element(&mut self,index:  usize) -> ForwardedElement {
         self.forwarded_element.remove(index)
     }
+    pub fn is_empty(&self) -> bool {
+        self.forwarded_element.is_empty()
+    }
+    pub fn from_utf8(v: &[u8]) -> Result<Self,ParseForwardedError> {
+        if let Ok(s) = std::str::from_utf8(v) {
+            return Self::from_str(s);
+        };
+        Err(ParseForwardedError)
+    }
+}
+
+use actix_web::http::header::{self,Header};
+use actix_web::error::ParseError;
+use actix_web::HttpMessage;
+use actix_web::http::HeaderName;
+use actix_web::http::header::IntoHeaderValue;
+use actix_web::http::header::InvalidHeaderValue;
+use actix_web::http::HeaderValue;
+use std::convert::TryFrom;
+
+impl IntoHeaderValue for Forwarded {
+    type Error = InvalidHeaderValue;
+    #[inline]
+    fn try_into(self) -> Result<HeaderValue, Self::Error> {
+        HeaderValue::try_from(self.to_string())
+    }
+}
+
+impl Header for Forwarded {
+    fn name() -> HeaderName {
+        header::FORWARDED
+    }
+    fn parse<T: HttpMessage>(msg: &T) -> Result<Self, ParseError> {
+        if let Some(hv) = msg.headers().get(Self::name()) {
+            return Ok(Self::from_utf8(hv.as_bytes())
+                .map_err(|_|ParseError::Header)?);
+        }
+        Ok(Forwarded::new())
+    }
 }
 //TODO. add more unexpected tests
 #[cfg(test)]
@@ -448,4 +528,9 @@ mod tests {
             ).unwrap();
         assert_eq!(f1,f2);
     }
+    #[test]
+    fn empty_forwarded() {
+        let f = Forwarded::from_str("").unwrap();
+        assert_eq!(f.is_empty(),true);
+    }
 }
